-- if player class is not hunter do not load anything
if (select(2, UnitClass("player"))) ~= "HUNTER" then return end

local L = AceLibrary("AceLocale-2.2"):new("SimpleMD")
local PlayerName = UnitName("player")
-- to be used soon because the hunters in my guild are lazy and I forsee version checks being needed
local revision = "$Revision: 45145 $"

local opts = {
	type='group',
	args = {
		messages = {
			type = 'group',
			name = L["Message Text Options"],
			desc = L["Options for setting the various alert texts"],
			args = {
				gain = {
					type = 'text',
					name = L["Gain Message"],
					desc = L["Set Misdirection Gained Message: use <player> to autofill your name and <target> to autofill MD's target's name"],
					usage = L["<your message>"],
					get = "GetMessageGain",
					set = "SetMessageGain",
		        },
				fade = {
					type = 'text',
					name = L["Faded Message"],
					desc = L["Set Misdirection Faded Message: use <player> to autofill your name and <target> to autofill MD's target's name"],
					usage = L["<your message>"],
					get = "GetMessageFade",
					set = "SetMessageFade",
				},
			},
		},
		color = {
			type = 'color',
			name = L["Text Color"],
			desc = L["Color for the On Screen Display"],
			get = "getColor",
			set = "setColor",
		},
		broadcast = {
			type = 'group',
			name = L["Broadcast Options"],
			desc = L["Options for how and where the alerts are displayed"],
			args = {
				screen = {
					type = 'toggle',
					name = L["Show on Screen"],
					desc = L["Toggles the display of the alert message on screen"],
					get = "IsShowOnScreen",
					set = "ToggleShowOnScreen",
				},
				console = {
					type = 'toggle',
					name = L["Show in Chat"],
					desc = L["Toggles the display of the alerts as self messages"],
					get = "IsShowInChat",
					set = "ToggleShowInChat",
				},
				channel = {
					type = 'toggle',
					name = L["Show in Channel"],
					desc = L["Toggles broadcasting of alerts to a user defined channel"],
					get = "IsShowInChannel",
					set = "ToggleShowInChannel",
				},
				floating = {
					type = 'toggle',
					name = L["Show in Combat Text"],
					desc = L["Toggles showing alerts in popular SCT like mods"],
					get = "IsShowFloating",
					set = "ToggleShowFloating",
				},
				showSelf = {
					type = 'toggle',
					name = L["Show Own Alerts"],
					desc = L["Toggles showing your alerts on your client"],
					get = "IsShowSelf",
					set = "ToggleShowSelf",
				},
				channelName = {
					type = 'text',
					name = L["Custom Channel"],
					desc = L["Name of custom channel"],
					usage = L["<channel name>"],
					get = "GetChannel",
					set = "SetChannel",
				},
			},
		},
		cooldowns = {
			type = 'group',
			name = L["Cooldown Broadcast Options"],
			desc = L["Toggle which cooldown warnings you want to display to other users"],
			args = {
				ready = {
					type = 'toggle',
					name = L["CD Finished Warning"],
					desc = L["Display a warning when MD cooldown finishes"],
					get = "IsReady",
					set = "ToggleReady",
				},
				ten = {
					type = 'toggle',
					name = L["10 Second Warning"],
					desc = L["Display a warning with 10 seconds left on MD cooldown"],
					get = "IsTenSeconds",
					set = "ToggleTenSeconds",
				},
				thirty = {
					type = 'toggle',
					name = L["30 Second Warning"],
					desc = L["Display a warning with 30 seconds left on MD cooldown"],
					get = "IsThirtySeconds",
					set = "ToggleThirtySeconds",
				},
				sixty = {
					type = 'toggle',
					name = L["60 Second Warning"],
					desc = L["Display a warning with 60 seconds left on MD cooldown"],
					get = "IsSixtySeconds",
					set = "ToggleSixtySeconds",
				},
			},
		},
		test = {
			type = 'execute',
			name = L["Test Message Settings"],
			desc = L["Prints a test message(s) to the screen, chat frame, and/or channel"],
			func = "messageTest",
		},
	},
}

SimpleMD = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceComm-2.0", "Sink-1.0")
SimpleMD:RegisterChatCommand(L["Slash-Commands"], opts)
SimpleMD:SetCommPrefix("SimpleMD")

SimpleMD:RegisterDB("SimplemdDB", "SimplemdDBPC")
SimpleMD:RegisterDefaults("profile", {
	messageGain = L["<player> cast Misdirection on <target>"],
	messageFade = L["Misdirection fades from <target>"],
	r = 1.0,
	g = 1.0,
	b = 1.0,
	channel = "",
	showInChannel = false,
	showInChat = false,
	showOnScreen = true,
	showFloating = false,
	showSelf = true,
	ready = true,
	ten = true,
	thirty = true,
	sixty = false,
})

function SimpleMD:OnEnable()
	self:RegisterComm("SimpleMD", "GROUP", "OnCommReceive")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD1")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD2")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD3")
	self:RegisterEvent("SIMPLEMD_MISDIRECT_CD4")
end

function SimpleMD:OnCommReceive(prefix, sender, distribution, message)	
	if self.db.profile.showInChat then
		self:Print(message)
	end
	if self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	end
	if self.db.profile.showFloating then
		self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
	end
end

function SimpleMD:SendMessage(message, id)	
	SimpleMD:SendCommMessage("GROUP", message)
	
	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChannel and self.db.profile.channel ~= "" then
			SendChatMessage(message, "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
		end
	
		if self.db.profile.showFloating and self.db.profile.showSelf and id ~= 1 then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
	end
end

--check to see if spell is Misdirection, if it is set the spell target
function SimpleMD:UNIT_SPELLCAST_SENT(arg1, arg2, arg3, arg4)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self.SpellTarget = arg4
		end
	end
end

-- if spellcast succeeds call the gain function and schedule cooldown events
-- needed to make sure messages are not sent when target is out of LOS or out of range or invalid
function SimpleMD:UNIT_SPELLCAST_SUCCEEDED(arg1, arg2, arg3)
	if arg1 == "player" then
		if arg2 == L["Misdirection"] then
			self:Gain()
			if self.db.profile.sixty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD1") --60 second warning
			end
			if self.db.profile.thirty then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD2") --30 second warning
			end
			if self.db.profile.ten then
				self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD3") -- 10 second warning
			end
			if self.db.profile.ready then
			self:ScheduleEvent("SIMPLEMD_MISDIRECT_CD4") -- cooldown finished
			end
		end
	end
	
	--account for the readiness talent
	if arg1 == "player" and arg2 == L["Readiness"] then
		self:CancelAllScheduledEvents()
		local temp = PlayerName .. " " .. L["used Readiness! MD ready!"]
		self:SendMessage(temp)
	end
end

function SimpleMD:CHAT_MSG_SPELL_AURA_GONE_SELF(arg1)
	if arg1 == L["Misdirection fades from you."] then
		self:Fade()
	end
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD1()
	local temp = PlayerName .. " " .. L["has 60 seconds left on MD cooldown."]
	self:SendMessage(temp, 2)
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD2()
	local temp = PlayerName .. " " .. L["has 30 seconds left on MD cooldown."]
	self:SendMessage(temp, 2)
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD3()
	local temp = PlayerName .. " " .. L["has 10 seconds left on MD cooldown!"]
	self:SendMessage(temp, 2)
end

function SimpleMD:SIMPLEMD_MISDIRECT_CD4()
	local temp = PlayerName .. " " .. L["has MD ready!"]
	self:SendMessage(temp, 2)
end

-- called on successful MD casts
function SimpleMD:Gain()
	local message = string.gsub(self.db.profile.messageGain, "<player>", PlayerName)
	message = string.gsub(message, "<target>", self.SpellTarget)

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		end
		if self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
		if self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		
		self:SendMessage(message, 1)
	end
end

-- called on Misdirection fading from self
function SimpleMD:Fade()
	local message = string.gsub(self.db.profile.messageFade, "<player>", PlayerName)
	message = string.gsub(message, "<target>", self.SpellTarget)

	if GetNumPartyMembers() > 0 or GetNumRaidMembers() > 0 then
		if self.db.profile.showInChat and self.db.profile.showSelf then
			self:Print(message)
		end
		if self.db.profile.showOnScreen and self.db.profile.showSelf then
			UIErrorsFrame:AddMessage(message, self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
		end
		if self.db.profile.showFloating and self.db.profile.showSelf then
			self:Pour(message, self.db.profile.r, self.db.profile.g, self.db.profile.b)
		end
		
		self:SendMessage(message, 1)
	end
end

-- test broadcast for making sure colors or channel is correct
function SimpleMD:messageTest()
	if self.db.profile.showInChat then
		self:Print(L["Testing Chat Frame Printing"])
	end
	if self.db.profile.showOnScreen then
		UIErrorsFrame:AddMessage(L["Testing On Screen Printing"], self.db.profile.r, self.db.profile.g, self.db.profile.b, 5.0)
	end
	if self.db.profile.showInChannel and channel ~= "" then
		SendChatMessage(L["Testing Channel Printing"], "CHANNEL", GetDefaultLanguage("player"), GetChannelName(self.db.profile.channel))
	end
	if self.db.profile.showFloating then
		self:Pour(L["Testing Floating Message"], self.db.profile.r, self.db.profile.g, self.db.profile.b)
	end
end

-- The many functions for the options menu
-- All the usual get/set pairs
function SimpleMD:GetMessageGain()
	return self.db.profile.messageGain
end

function SimpleMD:SetMessageGain(newMessage)
	self.db.profile.messageGain = newMessage
end

function SimpleMD:GetMessageFade()
	return self.db.profile.messageFade
end

function SimpleMD:SetMessageFade(newMessage)
	self.db.profile.messageFade = newMessage
end

function SimpleMD:IsShowInChat()
	return self.db.profile.showInChat
end

function SimpleMD:ToggleShowInChat()
	self.db.profile.showInChat = not self.db.profile.showInChat
end

function SimpleMD:IsShowOnScreen()
	return self.db.profile.showOnScreen
end

function SimpleMD:ToggleShowOnScreen()
	self.db.profile.showOnScreen = not self.db.profile.showOnScreen
end

function SimpleMD:getColor()
	return self.db.profile.r, self.db.profile.g, self.db.profile.b
end

function SimpleMD:setColor(r, g, b)
	self.db.profile.r, self.db.profile.g, self.db.profile.b = r, g, b
end

function SimpleMD:IsShowInChannel()
	return self.db.profile.showInChannel
end

function SimpleMD:ToggleShowInChannel()
	self.db.profile.showInChannel = not self.db.profile.showInChannel
end

function SimpleMD:GetChannel()
	return self.db.profile.channel
end

function SimpleMD:SetChannel(new)
	self.db.profile.channel = new
end

function SimpleMD:IsTenSeconds()
	return self.db.profile.ten
end

function SimpleMD:ToggleTenSeconds()
	self.db.profile.ten = not self.db.profile.ten
end

function SimpleMD:IsThirtySeconds()
	return self.db.profile.thirty
end

function SimpleMD:ToggleThirtySeconds()
	self.db.profile.thirty = not self.db.profile.thirty
end

function SimpleMD:IsSixtySeconds()
	return self.db.profile.sixty
end

function SimpleMD:ToggleSixtySeconds()
	self.db.profile.sixty = not self.db.profile.sixty
end

function SimpleMD:IsShowFloating()
	return self.db.profile.showFloating
end

function SimpleMD:ToggleShowFloating()
	self.db.profile.showFloating = not self.db.profile.showFloating
end

function SimpleMD:IsShowSelf()
	return self.db.profile.showSelf
end

function SimpleMD:ToggleShowSelf()
	self.db.profile.showSelf = not self.db.profile.showSelf
end

function SimpleMD:IsReady()
	return self.db.profile.ready
end

function SimpleMD:ToggleReady()
	self.db.profile.ready = not self.db.profile.ready
end
	